/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License")
 * with Classpath Exception; you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3ce
 *         (GPL 3 with Classpath Exception)
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation. cedarsoft GmbH designates this
 * particular file as subject to the "Classpath" exception as provided
 * by cedarsoft GmbH in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.gdao;

import com.cedarsoft.cache.Cache;
import com.cedarsoft.cache.HashedCache;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.lang.Override;
import java.util.Iterator;

/**
 * Manages the DAOs
 */
public class DefaultGenericDaoManager implements GenericDaoManager {
  @SuppressWarnings( {"MismatchedQueryAndUpdateOfCollection"} )
  @NotNull
  private final Cache<DaoTypeDescriptor<Object>, GenericDao<Object>> daoCache;

  /**
   * Creates a new dao manager
   *
   * @param daoFactory the factory
   */
  public DefaultGenericDaoManager( @NotNull Cache.Factory<DaoTypeDescriptor<Object>, GenericDao<Object>> daoFactory ) {
    daoCache = new HashedCache<DaoTypeDescriptor<Object>, GenericDao<Object>>( daoFactory );
  }

  /**
   * Returns the dao
   *
   * @param type the type
   * @return the dao for the given type
   */
  @Override
  @NotNull
  public <T> GenericDao<T> getDao( @NotNull Class<T> type ) {
    return getDao( type, null );
  }

  @Override
  @NotNull
  public <T> GenericDao<T> getDao( @NotNull Class<T> type, @Nullable LockProvider<T> lockProvider ) {
    return ( GenericDao<T> ) daoCache.get( new DaoTypeDescriptor<T>( type, lockProvider ) );
  }

  @Override
  public void shutdown() {
    for ( Iterator<GenericDao<Object>> it = daoCache.values().iterator(); it.hasNext(); ) {
      GenericDao<Object> dao = it.next();
      dao.shutdown();
      it.remove();
    }
  }
}
