/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License")
 * with Classpath Exception; you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3ce
 *         (GPL 3 with Classpath Exception)
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation. cedarsoft GmbH designates this
 * particular file as subject to the "Classpath" exception as provided
 * by cedarsoft GmbH in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.gdao.caching;

import com.cedarsoft.gdao.GenericDao;
import com.cedarsoft.gdao.GenericDaoManager;
import com.cedarsoft.gdao.LockProvider;
import com.cedarsoft.cache.Cache;
import com.cedarsoft.cache.HashedCache;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.lang.Override;
import java.util.Iterator;

/**
 *
 */
public class CachingDaoManager implements GenericDaoManager {
  @NotNull
  private final GenericDaoManager backingDaoManager;

  @SuppressWarnings( {"MismatchedQueryAndUpdateOfCollection"} )
  private final Cache<Class<Object>, CachingDao<Object>> daoCache = new HashedCache<Class<Object>, CachingDao<Object>>( new Cache.Factory<Class<Object>, CachingDao<Object>>() {
    @Override
    @NotNull
    public CachingDao<Object> create( @NotNull Class<Object> key ) {
      GenericDao<Object> backingDao = backingDaoManager.getDao( key );
      CachingDao<Object> dao = new CachingDao<Object>( backingDao );
      dao.initializeCache( backingDao.findAll() );
      return dao;
    }
  } );

  public CachingDaoManager( @NotNull GenericDaoManager backingDaoManager ) {
    this.backingDaoManager = backingDaoManager;
  }

  @Override
  @NotNull
  public <T> GenericDao<T> getDao( @NotNull Class<T> type ) {
    return ( GenericDao<T> ) daoCache.get( type );
  }

  @Override
  @NotNull
  public <T> GenericDao<T> getDao( @NotNull Class<T> type, @Nullable LockProvider<T> lockProvider ) {
    if ( lockProvider == null ) {
      return getDao( type );
    } else {
      throw new UnsupportedOperationException( "Lock provider not supported" );
    }
  }

  @Override
  public void shutdown() {
    for ( Iterator<CachingDao<Object>> it = daoCache.values().iterator(); it.hasNext(); ) {
      CachingDao<Object> dao = it.next();
      dao.shutdown();
      it.remove();
    }
    backingDaoManager.shutdown();
  }
}
